#!/usr/bin/env python
#
# Copyright 2018, Lenovo
#
# Time: 2018/03/07
#

from os import path, sep 
from argparse import ArgumentParser, HelpFormatter
from tools import NagiosPlugin 
from tools import Check_MKPlugin 

version = "1.0"
last_modify = "2018-10-12"


class my_ArgParser(ArgumentParser):
	"""
	This class just add some informations(version,copyright...) in print_help
	"""
	
	def print_help(file=None):
		"""
		Print version, copyright, last modify and license, then call ArgumentParse's print_help
		"""
		Head_info = "\nVersion: %s\nCopyright (c) 2018-present Lenovo Corp. \nLast Modified: %s\nLicense: GPL\n" % (version, last_modify)
		print(Head_info)
		ArgumentParser.print_help(file)


def parse_argument():
	"""
	This function just deal with the argument
	"""
	descript_msg = "This program is designed to install Lenovo ThinkSystem plugin to Nagios or Check_MK."
	note_msg = "Visit the website at https://exchange.nagios.org/directory/Plugins/Hardware/Server-Hardware/Lenovo/"
	parser = my_ArgParser(description = descript_msg, epilog = note_msg)
	parser.add_argument(
		"-c", "--check_mk", help="Automatic installation for check_mk (Vanilla Nagios is default option)", action="store_true")
	parser.add_argument(
		"-f", "--configfile", help="Config file for active discovery, the template file is /opt/lenovo/lnvgy-sw-lxce-nagios-plugin/template/active_temp.cfg")
	parser.add_argument(
		"-b", "--passive_config", help="Config file for passive discovery, the template file is /opt/lenovo/lnvgy-sw-lxce-nagios-plugin/template/passive_temp.cfg,(This parameter is only supported by Nagios)")
	parser.add_argument(
		"-i", "--importhost", help="Import ThinkSystem machine via the config file, the template file is same with '-f'. Unlike '-f', it doesn't require discovery, but adds all the host to Nagios/Check_MK config file")
	parser.add_argument(
		"-a", "--activediscovery", help="Only run active discovery", action="store_true")
	parser.add_argument(
		"-p", "--passivediscovery", help="Only run passive discovery.(This parameter is only supported by Nagios)", action="store_true")

	arg = parser.parse_args()
	return arg


def get_work_path():
	"""
	Get the real path of /opt/lenovo/lnvgy-sw-lxce-nagios-plugin/install.py
	"""
	workdir = path.dirname(path.realpath(__file__)) + sep
	return workdir


if __name__ == "__main__":
	args = parse_argument()
	is_check_mk = False
	only_adiscovery = False
	only_pdiscovery = False
	importhost = ""
	config_file = ""
	passive_cfg = ""
	local_path = get_work_path()

	if args.check_mk:
		is_check_mk = True
	if args.configfile:
		config_file = path.abspath(args.configfile)
	if args.passive_config:
		passive_cfg = path.abspath(args.passive_config)
	if args.importhost:
		importhost = path.abspath(args.importhost)
	if args.activediscovery:
		only_adiscovery = True
	if args.passivediscovery:
		only_pdiscovery = True

	if is_check_mk:
		ngs_mk = Check_MKPlugin(local_path, config_file, "", only_adiscovery, importhost)
	else:
		ngs_mk = NagiosPlugin(local_path, config_file, "",only_adiscovery, importhost)

	if only_adiscovery:
		ngs_mk.active_discovery_site(config_file, importhost)
	elif only_pdiscovery:
		if is_check_mk:
			print("Can not run only passive discovery for Check_MK")
		else:
			ngs_mk.passive_discovery_site(passive_cfg)
	else:
		ngs_mk.install_site(config_file, passive_cfg, importhost)


