#!/usr/bin/env python
# Copyright (C) 2018 Lenovo Corporation
#
# Time: 2018/03/07
# Description:  class for discovery Lenovo ThinkSytem Server


import argparse
import subprocess
from multiprocessing.dummy import Pool as ThreadPool
from shutil import copy
from os import path, sep, walk
from common_lib import get_connected_iplist
from common_lib import get_group_dict_info

## ConfigParse is different from python2.7 and python3.5
from sys import version as python_version

class ActiveDiscovery:

	def active_discovery(self, raw_group_dict):
		"""
		Active discovery according to raw group dict infor, return a dict of Lenovo XCC
		"""
		
		print("Discovering systems, please wait...")

		all_data_dict = {}

		for group in raw_group_dict:
			print("    Discovering group '%s'." % group)
			
			ip_list, engineid_list, alias, user, authlevel, snmp_aprotocol, apassword, atag, snmp_pprotocol, ppassword, ptag, services_list = get_group_dict_info(raw_group_dict[group])

			## Get list of ip which is connected via ping
			connected_ip_list = get_connected_iplist(ip_list)
			if 0 == len(connected_ip_list):
				print("    0 out of %s are successfully discovered in group %s\n" % (len(ip_list), group))
				continue

			## Get list of snmpwalk options which is used for judge XCC machine
			## This code is used for thread map
			snmpwalk_options_list = self.__get_snmpwalk_options_list(connected_ip_list, authlevel, user, snmp_aprotocol, apassword, snmp_pprotocol, ppassword)

			## Get list of ip which is XCC's address
			xcc_ip_list = self.__get_xcc_list(snmpwalk_options_list)
			if 0 != len(xcc_ip_list):
				print("    " + str(xcc_ip_list))
				print("    %s out of %s are successfully discovered in group %s\n" % (len(xcc_ip_list), len(ip_list), group))
			else:
				print("    0 out of %s are successfully discovered in group %s\n" % (len(ip_list), group))
				continue

			tmp_dict = dict(ip=xcc_ip_list, alias=alias, user = user, authlevel=authlevel, apassword=apassword, atag=atag, snmpaprotocol=snmp_aprotocol, ppassword=ppassword, ptag=ptag, snmppprotocol=snmp_pprotocol, services = services_list)
			all_data_dict[group] = tmp_dict

		return all_data_dict


	def __get_snmpwalk_options_list(self, connected_list, authlevel, user, aprotocol, apassword, pprotocol, ppassword):
		"""
		Get snmpwalk options list, this function write for thread and map function
		"""
		options_list = []
		for ip in connected_list:
			if 'authpriv' == authlevel.lower():
				snmpwalk_options = " -l authPriv -u %s -a %s -A %s -x %s -X %s %s" % (user, aprotocol, apassword, pprotocol, ppassword, ip)
			elif 'authnopriv' == authlevel.lower():
				snmpwalk_options = " -l authNoPriv -u %s -a %s -A %s %s" % (user, aprotocol, apassword, ip)
			elif 'noauthnopriv' == authlevel.lower():
				snmpwalk_options = " -l noAuthNoPriv -u %s %s" % (user, ip)
			else:
				break
			options_list.append(snmpwalk_options)
		return options_list


	def __get_xcc_list(self, snmpwalk_options_list):
		"""
		Use multithread to get Lenovo XCC ip list
		"""
		pool = ThreadPool()
		if 0 == len(snmpwalk_options_list):
			return []
		results = pool.map(self.__ispurley, snmpwalk_options_list)
		xcc_list = [str for str in results if str != "notxcc"]
		pool.close()
		pool.join()
		return xcc_list


	def __ispurley(self, snmpwalk_options):
		"""
		This function will judgment the machine is purley,
		Run snmpwalk command to find Lenovo XCC machines
		"""
		try:
			ip = snmpwalk_options.split(" ")[-1]
		except:
			return "notxcc"
        ##Lenovo-XCC-MIB: Get the XCC version type 1.3.6.1.4.1.19046.11.1.3.7.0 
		cmd = "snmpwalk -v3 %s 1.3.6.1.4.1.19046.11.1.3.7.0 -t 0.5 " % snmpwalk_options
		run_watch = subprocess.Popen(
			cmd.split(), stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
		ret, err_msg = run_watch.communicate()

		if "INTEGER:" in ret:
			return ip
		else:
			return "notxcc"

if "__main__" == __name__:
	pass
	
