#!/usr/bin/env python
# Copyright (C) 2018 Lenovo Corporation
#
# Time: 2018/03/07
# Description:  class for check_mk plugin installation


from shutil import copy
from os import mkdir, path, sep, environ
from common_lib import set_hosts_mk
from shutil import copy
import subprocess
import pprint, sys

from sys import version as python_version
if -1 != python_version.split(" ")[0].find("2.7"):
	input = raw_input

from nagios_cmk_base import NagiosCmkBase
from common_lib import select_y_n
from parser_discovery_cfg import ParserDiscoveryCfg
from active_discovery import ActiveDiscovery
from common_lib import encrypt


service_def_str = """  ( {'service_description': u'%s', 'command_line': '%s'}, ['%s' ,'/' + FOLDER_PATH + '/+'], ALL_HOSTS),\n"""

host_groups_def_str = """  ( '%s', ['/' + FOLDER_PATH + '/+'], %s, {'description': u'%s'} ),\n"""

group_mk_str = """
if type(define_hostgroups) != dict:
     define_hostgroups = {}
define_hostgroups.update({'all': u'Everything'})
"""



class Check_MKPlugin(NagiosCmkBase):

	def __init__(self, local_path = "", config_file = "", importfile = "", actv_dscry = False, pstv_dscry = False):
		self.site_path = self.__get_omd_path()
		self.local_path = local_path
		self.config_file = config_file
		self.importfile = importfile
		self.actv_dscry = actv_dscry
		self.pstv_dscry = pstv_dscry


	#################        hosts.mk        ################
	#
	# Generate hosts.mk via wato_import.py
	#
	##########################################################
	# Generate hosts.mk
	def __add_hosts_mk(self, group_dict):
		csv_str = self.__get_csv_str(group_dict)
		hosts_path = set_hosts_mk(csv_str, self.local_path)
		if hosts_path:
			print "- %s/hosts.mk" % hosts_path
			print "- %s/.wato" % hosts_path
		else:
			print "- %stools/etc/hosts.mk" % self.local_path
			print "- %stools/etc/.wato" % self.local_path
		return hosts_path



	# Get csv format string for wato_import.py to generate file hosts.mk
	def __get_csv_str(self, group_dict):
		csv_str = ""
		#wato_foldername;hostname|tag1 tag2;host_alias;ipaddress|None"""
		tmp_str = """lenovo;%s|%s;%s;%s\n"""
		for group in group_dict:
			for ip in group_dict[group]["ip"]:
				host_name = group + "_" + ip
				csv_str += tmp_str % (host_name,group,host_name,ip)
			
		return csv_str



	##################      rules.mk        #################
	#
	# Generate rules.mk file for Check_MK, and copy it to wato
	#
	##########################################################
	# Create rules file into the path: storage_path
	def __create_rules_files(self, discover_dict, storage_path):
		host_group_str = "host_groups = [\n"
		customer_check_str = "custom_checks = [\n"
		for group in discover_dict:
			group_member = []
			group_info = discover_dict[group]
			for ip in group_info["ip"]:
				group_member.append(group+"_"+ip)
			host_group_str += host_groups_def_str % (group ,str(group_member), group_info["alias"])
			customer_check_str += self.__get_service_str(group_info, group)

		host_group_str += "] + host_groups\n"
		customer_check_str += "] \n\n\n"

		self.__write_rules_mk(customer_check_str + host_group_str, storage_path + "/rules.mk")



	# check_lenovo_xcc.py -H "$HOSTADDRESS$" -v 3 -u $ARG1$ -l authPriv -a SHA -p $ARG2$ -X DES -x $ARG3$ -m $ARG4$
	# Get the string of customer service, which will write to rules.mk file
	def __get_service_str(self, group_info, group_name):
		customer_check_str = ""
		if "noauthnopriv" == group_info["authlevel"].lower():
			service_command = 'check_lenovo_xcc.py -H "$HOSTADDRESS$"  -u %s -l noAuthNoPriv -m ' % group_info["user"]
		elif "authnopriv" == group_info["authlevel"].lower():
			if group_info["atag"]:
				pass
			else:
				group_info["apassword"], group_info["atag"] = encrypt(group_info["apassword"])
			service_command = 'check_lenovo_xcc.py -H "$HOSTADDRESS$"  -u %s -l authNoPriv -a %s -p %s --atag %s -m ' % (group_info["user"], group_info["snmpaprotocol"], group_info["apassword"], group_info["atag"])
		elif "authpriv" == group_info["authlevel"].lower():
			if group_info["atag"] and group_info["ptag"]:
				pass
			else:
				group_info["apassword"], group_info["atag"] = encrypt(group_info["apassword"])
				group_info["ppassword"], group_info["ptag"] = encrypt(group_info["ppassword"])
			service_command = 'check_lenovo_xcc.py -H "$HOSTADDRESS$"  -u %s -l authPriv -a %s -p %s --atag %s -X %s -x %s --ptag %s -m ' % (group_info["user"], group_info["snmpaprotocol"], group_info["apassword"], group_info["atag"], group_info['snmppprotocol'], group_info['ppassword'], group_info["ptag"])

		for service in group_info["services"]:
			customer_check_str += service_def_str % ("check " + service, service_command + service, group_name)
		
		return customer_check_str



	# Write the information of customer service and host group into rules.mk 
	def __write_rules_mk(self, rules_mk_str, rules_path_name):
		try:
			fh = open(rules_path_name, 'w+')
			fh.write(rules_mk_str)
			fh.close()

		except:
			print "Failed in writing rules.mk"



	# This function will create rules.mk and copy it to wato
	def __add_rules_mk(self, group_dict, host_path):
		self.__create_rules_files(group_dict, self.local_path + "tools/etc/")
		rules_path = self.local_path + "tools/etc/rules.mk"
		if host_path:
			print "- %s/rules.mk" % host_path
			copy(rules_path, host_path)
		else:
			print "- %s" % rules_path



	########################     hosttags.mk   #########################
	#
	# Load hosttags.mk; add new tags; save hostags.mk
	#
	####################################################################
	# Load hosttags.mk and modify it according to the discovery
	def __modify_hosttags(self, omd_path, group_dict):
		host_list, aux_list = self.__load_hosttags(omd_path)
		group_list = self.__get_group_list(group_dict)
		new_hosttags = []
		add_new_tag = True
		for member in host_list:
			if member[0] == "Lenovo_XCC":
				new_choice = self.__add_new_choice(group_list, member[2])
				new_hosttags.append(("Lenovo_XCC", u"/Lenovo ThinkSystem group", new_choice))
				add_new_tag = False
			else:
				new_hosttags.append(member)
		
		if add_new_tag:
			new_choice = self.__add_new_choice(group_list, [])
			new_hosttags.append(("Lenovo_XCC", u"/Lenovo ThinkSystem group", new_choice))

		self.__save_hosttags(new_hosttags, aux_list, omd_path)



	# Get the group list from result of discovery, is will compare with 
	# hosttags.mk's group list, so that we can add new group in hosttags.mk 
	# format: group_name, u"group_descript"
	def __get_group_list(self, discovery_dict):
		group_list = []
		for group in discovery_dict:
			descript = discovery_dict[group]['alias']
			tmp_group_info = (group, u"%s" % descript, [])
			group_list.append(tmp_group_info)
		return group_list



	# Merge old group and new group, then write it to hosttags.mk
	def __add_new_choice(self, group_list, old_choice):
		if old_choice:
			new_choice = old_choice
		else:
			new_choice = [("nogroup", u"not Lenovo ThinkSystem", []),]
		old_group_list = [x[0] for x in old_choice]
		for group in group_list:
			if group[0] not in old_group_list:
				new_choice.append(group)
		return new_choice



	###################################################################
	#
	# Add hostgroup define in group.mk
	#
	###################################################################
	def __add_hostgroup_def(self, site_path):
		hostgrp_def = False
		tmp_str = ""
		group_mk_file = site_path + "/etc/check_mk/conf.d/wato/groups.mk"
		with open(group_mk_file, "r") as f:
			for line in f:
				if 'define_hostgroups' in line:
					hostgrp_def = True
					return
				tmp_str += line
		if hostgrp_def:
			pass
		else:
			tmp_str += group_mk_str
			with open(group_mk_file, "w") as f:
				f.write(tmp_str)
			

	####################################################################
	#
	# The two function is come from wato's lib, which is used to load host tags
	#   and save host tags
	#
	#####################################################################
	# Load file hosttags.mk, return wato_host_tags and wato_aux_tags
	def __load_hosttags(self, omd_path):
		filename = omd_path + "/etc/check_mk/multisite.d/wato/hosttags.mk"
		if not path.exists(filename):
			return [], []
		try:
			vars = {
				"wato_host_tags" : [],
				"wato_aux_tags" : []}
			execfile(filename, vars, vars)
			# Convert manually crafted host tags tags WATO-style. This
			# makes the migration easier
			for taggroup in vars["wato_host_tags"]:
				for nr, entry in enumerate(taggroup[2]):
					if len(entry) <= 2:
						taggroup[2][nr] = entry + ([],)
			return vars["wato_host_tags"], vars["wato_aux_tags"]

		except Exception, e:
			print("Cannot read configuration file %s: %s") % (filename, e)
			return [], []



	# Save file hosttags.mk, according auxtags and hosttags 
	def __save_hosttags(self, hosttags, auxtags, omd_path):
		filename = omd_path + "/etc/check_mk/multisite.d/wato/hosttags.mk"
		output = "# Create by WATO\n# encoding: utf-8\n"
		output += "wato_host_tags += \\\n%s\n\n" % pprint.pformat(hosttags)
		output += "wato_aux_tags += \\\n%s\n" % pprint.pformat(auxtags)

		file(filename, "w").write(output) 



	#################################################################
	#
	# Call discovery function to get ThinkSystem machine's group dict 
	#
	################################################################
	def __get_group_dict(self, config_file, importfile = ""):
		#ngs_dscy = nagios_discovery()
		parser_cfg = ParserDiscoveryCfg()
		actv_discovery = ActiveDiscovery()
		ret_dict = dict()

		if config_file:
			pass
		else:
			# Get discovery config file path
			config_file = self._get_discovery_cfg_file(2)

		raw_group_dict = parser_cfg.get_raw_group_dict(config_file, True)
		active_dict = actv_discovery.active_discovery(raw_group_dict)
		
		# If point importfile, merge active_dict and import_dict
		if importfile:
			import_raw_group_dict = parser_cfg.get_raw_group_dict(importfile, True)
			ret_dict = self.merge_dict(active_dict, import_raw_group_dict)
		else:
			ret_dict = active_dict

			
		return ret_dict



	###################################################################
	#
	# Get omd path
	#
	##################################################################
	# Get omd path 
	def __get_omd_path(self):
		try:
			omd_path = environ['OMD_ROOT']
			tmp_path = path.expanduser("~/")
			if 'omd/sites' not in tmp_path:
				sys.exit()
		except:
			print "Run this script inside a OMD site for Check_MK"
			sys.exit()

		omd_path = omd_path + sep
		print "\nCheck_MK path is %s\n" % omd_path
		return omd_path

	def __install_plugin(self):
		"""	Install plugin 'check_lenovo_xcc.py' in to Nagios or Check_mk

		copy script 'check_lenovo_xcc.py' into $(OMD_path)/local/lib/nagios/plugins/

		"""
		local_path = self.local_path
		site_path = self.site_path

		print("Installing Plugin-In 'check_lenovo_xcc.py' ...")

		src_plugin = local_path + "tools/libexec/check_lenovo_xcc.py"
		dst_plugin = site_path + "local/lib/nagios/plugins/check_lenovo_xcc.py"
		
		if not path.exists(dst_plugin):
			copy(src_plugin, dst_plugin)
			print("    Copy file %s to %s" % (src_plugin, dst_plugin))
		else:
			if cmp(src_plugin, dst_plugin):
				print("    'check_lenovo_xcc.py' has been installed")
			else:
				select_val = select_y_n("    File %s exists, would you want to replace it? (y/n) (Blank is y)" % dst_plugin)
				if select_val.lower() == "y":
					copy(src_plugin, dst_plugin)
					print("    Copy file %s to %s" % (src_plugin, dst_plugin))
				else:
					print("    File check_lenovo_xcc.py is saved in %s, you can hand it manually.", local_path + "tools/libexec/")

	# Reboot nagios or cmk
	def __reboot_site(self):
		"""Restart Nagios or Check_MK with commands:
		run command 'cmk -R' to restart
		"""
		cmd = "cmk -R"
		select_val = select_y_n("Do you want to restart Check_MK? (y/n)(Blank is y)")
		
		if 'y' == select_val.lower():
			run_watch = subprocess.Popen(cmd.split(), stdout = subprocess.PIPE, stderr = subprocess.PIPE, universal_newlines=True)
			ret, err_msg = run_watch.communicate()
			print("Restart Messages:\n%s" % ret + err_msg)


	############################################################
	#
	# External interface
	#
	############################################################

	# install hosts.mk, rules.mk, and modify hosttags.mk
	def install_site(self, cfg_file, passive_cfg, importfile=""):
		omd_path = self.site_path
		local_path = self.local_path
		self.__install_plugin()

		if "y" == select_y_n("Do you wish to run active discovery? (y/n) (Blank is y)"):
			group_dict = self.__get_group_dict(cfg_file, importfile)
			self.__modify_hosttags(omd_path, group_dict)
			print "Configuration files are been generated..."
			host_path = self.__add_hosts_mk(group_dict)
			self.__add_rules_mk(group_dict, host_path)
			self.__add_hostgroup_def(omd_path)
		self.__reboot_site()


	def active_discovery_site(self, cfg_file, importfile):
		omd_path = self.site_path
		group_dict = self.__get_group_dict(cfg_file, importfile)
		self.__modify_hosttags(omd_path, group_dict)
		print "Configuration files are been generated..."
		host_path = self.__add_hosts_mk(group_dict)
		self.__add_rules_mk(group_dict, host_path)
		self.__add_hostgroup_def(omd_path)
		self.__reboot_site()


if "__main__" == __name__:

	only_adiscovery = False
	only_pdiscovery = False
	config_file = ""
	nagios_path = ""
	importfile = ""

	local_path = "../"
	h = Check_MKPlugin(local_path, config_file, importfile, only_adiscovery, False)

	#h.active_discovery_site(config_file, importfile)
	h.install_site(config_file, "", importfile)
