#!/usr/bin/env python
# Copyright (C) 2018 Lenovo Corporation
#
# Time: 2018/03/07
# Description:  basic tools for nagios plugin installation


import subprocess
from multiprocessing.dummy import Pool as ThreadPool
import os

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.ciphers import (
		    Cipher, algorithms, modes
			)
import warnings
warnings.filterwarnings("ignore")

from binascii import b2a_hex, a2b_hex


from sys import version_info as python_info
if python_info < (3, 0):
    lx_input = raw_input
else:
    lx_input = input


def get_encryption_key():
	uuid = ''
	test = subprocess.Popen(['cat', '/etc/fstab'], stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
	stdout,stderr = test.communicate()
	for i in stdout.split("\n"):
		if "boot" in i:
			for j in i.split(" "):
		  		if "UUID" in j:
					uuid = j
	return uuid

key = get_encryption_key()
key = key[:32]
associated_data = "authenticated but not encrypted payload"

## IP to number
def ip2num(ip):
	ip_segment = [int(x) for x in ip.split('.')]
	return ip_segment[0] << 24 | ip_segment[1] << 16 | ip_segment[2] << 8 | ip_segment[3]


## Number to ip
def num2ip(num):
	ip = ['', '', '', '']
	ip[3] = (num & 0xff)
	ip[2] = (num & 0xff00) >> 8
	ip[1] = (num & 0xff0000) >> 16
	ip[0] = (num & 0xff000000) >> 24
	return '%s.%s.%s.%s' % (ip[0], ip[1], ip[2], ip[3])


# Get ip list from 3 format: 
# format1: 198.126.0.0/24
# format2: 198.126.0.1-198.126.0.100
# format3: 198.126.0.1(just one ip address)
def getiplist_impl(ip_range):
	if -1 != ip_range.find("-"):
		index = ip_range.find("-")
		try:
			l_endpoint = ip2num(ip_range[:index])
			r_endpoint = ip2num(ip_range[index + 1:])
			if l_endpoint >= r_endpoint:
				print("The right endpoint must be greater than the left endpoint(%s)" % ip_range)
				return None
			else:
				ret_list = [num2ip(ip) for ip in range(l_endpoint, r_endpoint + 1)]
				return [ip for ip in ret_list if ip.split(".")[-1] !="0" and ip.split(".")[-1] != "255" ]
		except:
			print("IP range %s is invalid, please set a valid one in configuration file" % ip_range)
			return None
	elif -1 != ip_range.find("/"):
		index = ip_range.find("/")
		ip_addr = ip_range[:index]
		mask = ip_range[index + 1:]
		if 0 != ip2num(ip_addr) & (0xffffffff >> int(mask)):
			print("IP range %s is invalid, please set a valid one in configuration file" % ip_range)
			return None
		else:
			l_endpoint =  ip2num(ip_addr)
			r_endpoint = l_endpoint + (0xffffffff >> int(mask))
			return [num2ip(ip) for ip in range(l_endpoint, r_endpoint + 1)]
	else:
		return [ip_range]


## Get ip list from ip range
def getiplist(ip_str):
	iplist = []
	iprange_list = ip_str.split(",")
	iprange_list = [iprange.strip() for iprange in iprange_list if '' != iprange.strip()]
	for iprange in iprange_list:
		ret_iplist = getiplist_impl(iprange)
		if ret_iplist:
			iplist += ret_iplist
	return iplist


## Ping the IP to test connect state
def isconnect(ip):
	cmd = "ping %s -c 2 -W 1" % ip
	run_watch = subprocess.Popen(cmd.split(),
		stdout=subprocess.PIPE, stderr=subprocess.PIPE,universal_newlines=True)
	ret, err_msg = run_watch.communicate()
	if -1 == ret.find("100%") and -1 == ret.find("50%"):
		return ip
	else:
		return "notconnect"

## Use multithread to get connected ip list
def get_connected_iplist(ip_list):
	pool = ThreadPool()
	results = pool.map(isconnect, ip_list)
	ok_list = [str for str in results if str != "notconnect"]
	pool.close()
	pool.join()
	return ok_list

# Just for get input value y/n
def select_y_n(show_info):
	"""	Get input value y/n from user.

	You have 3 times to chose 'y' or 'n'
	input 'y', 'Y' and ''(blank) ==> ouput 'y'
	input 'n' and 'N' ==> output 'n'
	type error more than 3 times, output 'n'

	Args:
		[show_info]: print some messages to prompt customer
	"""
	for i in range(3):
		ret_value = lx_input("\n%s " % show_info)
		if "" == ret_value or 'y' == ret_value.lower():
			return 'y'
		elif 'n' == ret_value.lower():
			return 'n'
		else:
			print("  ERROR: You must type 'y' or 'n' ...")
	print("  Too many error inputs make 'n' selected!")
	
	return 'n'

def get_group_dict_info(group_dict):
	"""
	return:
		ip_list, engineid_list, alias, user, authlevel, snmp_aprotocol, apassword, snmp_pprotocol, ppassword, services_list
	"""
	ip_list, engineid_list, alias, user, authlevel, snmp_aprotocol, apassword, atag, snmp_pprotocol, ppassword, ptag, services_list = [],[],'','','','','',None,'','',None,[]

	if 'ip' in group_dict.keys():
		ip_list = group_dict['ip']

	if 'engineid' in group_dict.keys():
		engineid_list = group_dict['engineid']

	if 'alias' in group_dict.keys():
		alias = group_dict['alias']

	if 'user' in group_dict.keys():
		user = group_dict['user']

	if 'authlevel' in group_dict.keys():
		authlevel = group_dict['authlevel']

	if 'snmpaprotocol' in group_dict.keys():
		snmp_aprotocol = group_dict['snmpaprotocol']

	if 'apassword' in group_dict.keys():
		apassword = group_dict['apassword']

	if 'atag' in group_dict.keys():
		atag = group_dict['atag']

	if 'snmppprotocol' in group_dict.keys():
		snmp_pprotocol = group_dict['snmppprotocol']

	if 'ppassword' in group_dict.keys():
		ppassword = group_dict['ppassword']

	if 'ptag' in group_dict.keys():
		ptag = group_dict['ptag']

	if 'services' in group_dict.keys():
		services_list = group_dict['services']
	
	return ip_list, engineid_list, alias, user, authlevel, snmp_aprotocol, apassword, atag, snmp_pprotocol, ppassword, ptag, services_list


def encrypt(plaintext):
	# Generate a random 96-bit IV.
	iv = key[:12]

	# Construct an AES-GCM Cipher object with the given key and a
	# randomly generated IV.
	encryptor = Cipher(
		algorithms.AES(key),
		modes.GCM(iv),
		backend=default_backend()
	).encryptor()

	# associated_data will be authenticated but not encrypted,
	# it must also be passed in on decryption.
	encryptor.authenticate_additional_data(associated_data)

	# Encrypt the plaintext and get the associated ciphertext.
	# GCM does not require padding.
	ciphertext = encryptor.update(plaintext) + encryptor.finalize()

	return (b2a_hex(ciphertext), b2a_hex(encryptor.tag))


def decrypt(ciphertext, tag = None):
	iv = key[:12]
	# Construct a Cipher object, with the key, iv, and additionally the
	# GCM tag used for authenticating the message.
	decryptor = Cipher(
		algorithms.AES(key),
		modes.GCM(iv, a2b_hex(tag)),
		backend=default_backend()
	).decryptor()

	# We put associated_data back in or the tag will fail to verify
	# when we finalize the decryptor.
	decryptor.authenticate_additional_data(associated_data)

	# Decryption gets us the authenticated plaintext.
	# If the tag does not match an InvalidTag exception will be raised.
	return decryptor.update(a2b_hex(ciphertext)) + decryptor.finalize()


if  __name__ == "__main__":
	
	ciphertext, tag = encrypt(
		b"a secret message!"
	)

	print ciphertext
	print tag
	
	print(decrypt(
		ciphertext,
		tag
	))	


