#!/usr/bin/env python
# Copyright (C) 2018 Lenovo Corporation
#
# Time: 2018/07/05
# Description: handle passive discovery via snmp-trap


import os,sys
from sys import exit
from os import path
import argparse
import subprocess
import json

## ConfigParse is different from python2.7 and python3.5
from sys import version as python_version
if -1 != python_version.split(" ")[0].find("2.7"):
	import ConfigParser
	ConfigParser = ConfigParser.ConfigParser
else:
	import configparser
	ConfigParser = configparser.ConfigParser

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.ciphers import (
		    Cipher, algorithms, modes
			)
import warnings
warnings.filterwarnings("ignore")

from binascii import b2a_hex, a2b_hex

def get_encryption_key():
	uuid = ''
	test = subprocess.Popen(['cat', '/etc/fstab'], stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
	stdout,stderr = test.communicate()
	for i in stdout.split("\n"):
		if "boot" in i:
			for j in i.split(" "):
		  		if "UUID" in j:
					uuid = j
	return uuid

key = get_encryption_key()
key = key[:32]
associated_data = "authenticated but not encrypted payload"


host_note = """
###############################################################################
###############################################################################
#
# HOST DEFINITIONS
#
###############################################################################
###############################################################################

# Define hosts for the Lenovo XCC machines we'll be monitoring
# Change the host_name, alias, and address to fit your situation
"""

group_note = """
###############################################################################
###############################################################################
#
# HOST GROUP DEFINITIONS
#
###############################################################################
###############################################################################

# Define hostgroups for Lenovo XCC machines
# All hosts that use the linux-server template will automatically be a member of this group
"""

service_note = """
###############################################################################
###############################################################################
#
# SERVICE DEFINITIONS
#
###############################################################################
###############################################################################

# Create services for monitoring
"""

host_group_tmp = """
define hostgroup{
	hostgroup_name    %s  ; The name of the hostgroup
	alias             %s  ; Long name of the group
	members           %s  ; Members in this group
}
"""

service_tmp = """
define service{
	use                 generic-service
	hostgroup_name      %s
	service_description Check %s 
	check_command       %s
}
"""

alert_service_tmp = """
define service{
	use                 generic-service
	hostgroup_name      %s
	active_checks_enabled           0
	passive_checks_enabled          1
	service_description %s 
	check_command       check-host-alive
	contact_groups      lnv_admins
	max_check_attempts              1
	retry_check_interval            1
	is_volatile                     1
	#obsess_over_service            0
}
"""

host_tmp = """
define host{
    use             linux-server    ; Inherit default values from a template
    host_name       %s ; The name we're giving to this host
    alias           %s ; A longer name associated with the host
    address         %s  ; IP address of the host
}
"""


class nagios_discovery:

	def load_config_file(self, cfg_file):
		"""
		load nagios config file which defined host,services and groups, return XCC dict
		"""
		list_hostgroup = []
		list_service = []
		list_host = []
		
		if not path.isfile(cfg_file):
			return {} 

		with open(cfg_file) as f:
			for line in f:
				if line.strip() == "" or line.strip()[0] == "#" :
					continue
				elif ';' in line:
					line_tmp = line[:line.find(";")] + "\n"
				else:
					line_tmp = line
	
				# 1. Get the data segment in 'define xxx{' and '}'
				# 2. Set the data_type according the define name "xxx", and get the lists of hostgroup and service dict
				# 3. Get the full dict according the lists of hostgroup and service dict
				# 4. Note: the host list is not used
				if "define host{" in line_tmp or "define service{" in line_tmp or "define hostgroup{" in line_tmp:
					cfg_data_segment = line_tmp
				elif "}" == line_tmp.strip():
					cfg_data_segment += line_tmp
					data_type,dict_info = self.__handle_cfg_data_segment(cfg_data_segment)
					if "hostgroup" == data_type:
						list_hostgroup.append(dict_info)
					elif "service" == data_type:
						list_service.append(dict_info)
					elif "host" == data_type:
						list_host.append(dict_info)
					else:
						pass
				else:
					cfg_data_segment += line_tmp
			
			dict_return = self.__get_dict_via_group_service(list_hostgroup, list_service)
		return dict_return


	def __handle_cfg_data_segment(self, cfg_data_segment):
		"""
		according to the config file segment,
		return the data type and a dict for this data
		"""
		data_type = "unknown"
		dict_return = {}
		cfg_infor = cfg_data_segment.split('\n')
		if 0 == len(cfg_infor):
			return data_type, dict_return
		if 'hostgroup' in cfg_infor[0]:
			data_type = 'hostgroup'
		elif 'service' in cfg_infor[0]:
			data_type = 'service'
		elif 'host' in cfg_infor[0]:
			data_type = 'host'
		else:
			return data_type, dict_return

		# Get the dict which key is the string before first space, and others string is value
		# eg. "host test1", the key is 'host' and the value is 'test1'
		for x in range(1, len(cfg_infor)-2):
			tmp_str = cfg_infor[x].strip()
			space_index = tmp_str.find(" ")
			dict_key = tmp_str[:space_index].strip()
			dict_value = tmp_str[space_index:].strip()
			dict_return[dict_key] = dict_value

		return data_type, dict_return


	def __get_dict_via_group_service(self, list_hostgroup, list_service):
		"""
		Get the full dict according the lists of group and services
		"""
		dict_return = {}

		# Get the dict key 'group_name', members and alias according to hostgroup list
		for group in list_hostgroup:
			tmp_dict={}
			group_name = group['hostgroup_name']
			tmp_members_list = group['members'].split(',')
			group_members_list = [ x.strip().split('_')[-1] for x in tmp_members_list]
			tmp_dict["ip"] = group_members_list
			tmp_dict["alias"] = group['alias']
			dict_return[group_name] = tmp_dict

		# Get the other information, eg. authlevel, userid, password and soon
		for service in list_service:
			tmp_dict = {}
			group_name = service['hostgroup_name']
			if group_name in dict_return:
				service_info = service['check_command'].split('!')
				if service_info[0] == "check_system_authPriv" and len(service_info) == 9 :
					tmp_dict['authlevel'] = "authpriv"
					tmp_dict['user'] = service_info[1]
					tmp_dict['snmpaprotocol'] = service_info[2]
					tmp_dict['apassword'] = service_info[3]
					tmp_dict['atag'] = service_info[4]
					tmp_dict['snmppprotocol'] = service_info[5]
					tmp_dict['ppassword'] = service_info[6]
					tmp_dict['ptag'] = service_info[7]
					tmp_dict['services'] = [service_info[8]]
				elif service_info[0] == "check_system_authNoPriv" and len(service_info) == 6 :
					tmp_dict['authlevel'] = "authnopriv"
					tmp_dict['user'] = service_info[1]
					tmp_dict['snmpaprotocol'] = service_info[2]
					tmp_dict['apassword'] = service_info[3]
					tmp_dict['atag'] = service_info[4]
					tmp_dict['services'] = [service_info[5]]
				elif service_info[0] == "check_system_noAuthNoPriv" and len(service_info) == 3 :
					tmp_dict['authlevel'] = "noauthnopriv"
					tmp_dict['user'] = service_info[1]
					tmp_dict['services'] = [service_info[2]]
				# This case is set for passive alert services, critical/non-critical/system/recovery
				elif service_info[0] == "check-host-alive":
					tmp_dict['services'] = ["passive_alert"]
					
				dict_return[group_name] = self._merge_common_dict(dict_return[group_name], tmp_dict)
		
		return dict_return


	def merge_dict(self, cfg_dict, discovery_dict):
		"""
		Merge the dict get from discovery and get from load nagios config file
		  return: add discovery host into old host dict, but None means that discovery host is already existed
		"""
		if not (cfg_dict and discovery_dict):
			cfg_dict.updata(discovery_dict)
			return cfg_dict
		else:
			for key in discovery_dict:
				# if the discovery host is already existed.
				if key in cfg_dict:
					return None
				else:
					cfg_dict[key]=discovery_dict[key]
			return cfg_dict


	def _merge_common_dict(self, dict1, dict2):
		"""
		Merge dict1 and dict2, and merge lists "ip" and "service"
		"""
		for key in dict1:
			if (key in dict2) and (key in ["ip", "services"]):
				dict1[key] = list(set(dict1[key] + dict2[key]))
		for key in dict2:
			if key not in dict1:
				dict1[key] = dict2[key]
		return dict1


	def get_group_dict_via_host(self, host_ip, discover_cfg):
		"""
		Get group dict information via host ip which get from snmptrap
		"""
		tmp_str = ""
		with open(discover_cfg, "r") as f:
			tmp_str = f.readlines()
		
		tmp_str = "".join(tmp_str)
		group_list = json.loads(tmp_str)
		
		group_dict = {}

		for group, group_data in group_list.items():

			items = group_data

			ip_range, engineid, alias, user, authlevel, snmp_aprotocol, apassword, atag, snmp_pprotocol, ppassword, ptag = \
					self._get_user_info_via_cfg(items)
			services_list = []
			if "services" in items.keys():
				services_list = items['services']
				services_list = [service.strip() for service in services_list if '' != service.strip()]
				services_list.append("passive_alert")
			else:
				services_list = ['all', 'power', 'cpu', 'storage', 'temperature', 'fans', 'voltage', 'memory', 'passive_alert']


			if engineid:
				if authlevel.lower() == "authpriv":
					if atag and ptag:
						apassword = decrypt(apassword, atag)
						ppassword = decrypt(ppassword, ptag)

					snmpwalk_options = " -l %s -u %s -a %s -A %s -x %s -X %s %s " % (authlevel, user, snmp_aprotocol, apassword, snmp_pprotocol, ppassword, host_ip)
				elif authlevel.lower() == "authnopriv":
					snmpwalk_options = " -l %s -u %s -a %s -A %s %s " % (authlevel, user, snmp_aprotocol, apassword, host_ip)
				elif authlevel.lower() == "noauthnopriv":
					snmpwalk_options = " -l %s -u %s  %s " % (authlevel, user, host_ip)
				else:
					continue
				
				if "notxcc" != self.__ispurley(snmpwalk_options):
					tmp_dict = dict(ip=[host_ip], alias=alias, user = user, authlevel=authlevel, apassword=apassword, snmpaprotocol=snmp_aprotocol, ppassword=ppassword, snmppprotocol=snmp_pprotocol, services = services_list)
					group_dict[group] = tmp_dict

		return group_dict


	def _get_user_info_via_cfg(self, group_dict):
		"""
		return:
			ip_range, engineid, alias, user, authlevel, snmp_aprotocol, apassword, snmp_pprotocol, ppassword
		"""
		ip_range, engineid, alias, user, authlevel, snmp_aprotocol, apassword, atag, snmp_pprotocol, ppassword, ptag = '','','','','','','',None,'','', None

		if 'ip' in group_dict.keys():
			ip_range = group_dict['ip']

		if 'engineid' in group_dict.keys():
			engineid = group_dict['engineid']

		if 'alias' in group_dict.keys():
			alias = group_dict['alias']

		if 'user' in group_dict.keys():
			user = group_dict['user']

		if 'authlevel' in group_dict.keys():
			authlevel = group_dict['authlevel']

		if 'snmpaprotocol' in group_dict.keys():
			snmp_aprotocol = group_dict['snmpaprotocol']

		if 'apassword' in group_dict.keys():
			apassword = group_dict['apassword']

		if 'atag' in group_dict.keys():
			atag = group_dict['atag']

		if 'snmppprotocol' in group_dict.keys():
			snmp_pprotocol = group_dict['snmppprotocol']

		if 'ppassword' in group_dict.keys():
			ppassword = group_dict['ppassword']

		if 'ptag' in group_dict.keys():
			ptag = group_dict['ptag']
		
		return ip_range, engineid, alias, user, authlevel, snmp_aprotocol, apassword,atag, snmp_pprotocol, ppassword, ptag


	def __ispurley(self, snmpwalk_options):
		"""
		This function will judgment the machine is purley,
		Run snmpwalk command to find Lenovo XCC machines
		"""
		try:
			ip = snmpwalk_options.split(" ")[-1]
		except:
			return "notxcc"
        ##Lenovo-XCC-MIB: Get the XCC version type 1.3.6.1.4.1.19046.11.1.3.7.0 
		cmd = "snmpwalk -v3 %s 1.3.6.1.4.1.19046.11.1.3.7.0 -t 0.5 " % snmpwalk_options
		run_watch = subprocess.Popen(
			cmd.split(), stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
		ret, err_msg = run_watch.communicate()
		if "INTEGER:" in ret:
			return ip
		else:
			return "notxcc"


	def save_config_file(self, dict_full, config_path):
		"""
		Save config file according to dict which include host, group and servers
		"""
		config_str = self.__dict2str(dict_full)
		return self.__write_config_file(config_str, config_path)

	
	def __dict2str(self, dict_full):
		"""
		Get the config file string according the dict information 
		"""
		host_cfg_str = ""
		group_cfg_str = ""
		services_cfg_str = ""
		for group, group_data in dict_full.items():
			print group_data
			services_list = group_data['services']
			if 0 == len(services_list):
				services_list = ['all', 'power', 'cpu', 'storage', 'temperature', 'fans', 'voltage', 'memory']
			xcc_ip_list, engineid, alias, user, authlevel, snmp_aprotocol, apassword, atag, snmp_pprotocol, ppassword, ptag = self._get_user_info_via_cfg(group_data)
		
			## Get the host config
			for xcc_ip in xcc_ip_list:
				host_cfg_str += host_tmp % (group+'_'+xcc_ip, group+'_'+xcc_ip, xcc_ip)

			## Get the group config
			## Get the members of this group
			members_list = [group +"_" + xcc_ip for xcc_ip in xcc_ip_list]
			group_members = ','.join(members_list)
			if group_members:
				group_cfg_str += host_group_tmp % (group, alias, group_members)

			## Get the services config
			## get command for services
			if 'authpriv' == authlevel.lower():
				if '' == apassword or '' == ppassword:
					print("Both of authentication and privacy protocol password are necessary for authPriv level")
					continue
				else:
					if atag and ptag:
						pass
					else:
						apassword, atag = encrypt(apassword)
						ppassword, ptag = encrypt(ppassword)

					command_str = "%s!%s!%s!%s!%s!%s!%s!%s" % ("check_system_authPriv", user, snmp_aprotocol, apassword, atag, snmp_pprotocol, ppassword, ptag)
			elif 'authnopriv' == authlevel.lower():
				if '' == apassword:
					print("Authentication protocol password is necessary for authPriv level")
					continue
				else:
					if atag:
						pass
					else:
						apassword = encrypt(apassword, atag)
					command_str = "%s!%s!%s!%s!%s" % ("check_system_authNoPriv", user, snmp_aprotocol, apassword, atag)
			elif 'noauthnopriv' == authlevel.lower():
				command_str = "%s!%s" % ("check_system_noAuthNoPriv", user)

			## add services 
			if services_list:
				services_cfg_str += ("\n#### Services for %s ####\n" % group)
			has_passive_alert = False
			for service in services_list:
				if "passive_alert" == service:
					has_passive_alert = True
					continue
				services_cfg_str += service_tmp % (group, service, command_str + "!" + service)
			if has_passive_alert:
				#alert_descript_list = ["Critical Alert", "Non-Critical Alert", "System Alert", "Recovery"]
				alert_descript_list = ["Critical Alert", "System Alert"]
				for alert_str in alert_descript_list:
					services_cfg_str += alert_service_tmp % (group, alert_str)

		return host_note + host_cfg_str + group_note + group_cfg_str + service_note + services_cfg_str


	def __write_config_file(self, config_str, config_file):
		"""
		Write the config file to config file, config_file_name = path + file name
		"""
		try:
			fh = open(config_file, 'w+')
			fh.write(config_str)
			fh.close()
		except:
			print("Failed in writing configuration file")
			return False
		return True



def get_trap_info():
	ip_str = ""
	for sysline in sys.stdin:
		if "UDP:" in sysline:
			first_index = sysline.find("[")
			second_index = sysline.find("]")
			ip_str = sysline[first_index + 1:second_index]
	return ip_str


def reboot_nagios():
	"""
	Run command 'service nagios restart' to restart Nagios
	"""
	cmd = "service nagios restart"
	run_watch = subprocess.Popen(cmd.split(), stdout = subprocess.PIPE, stderr = subprocess.PIPE, universal_newlines=True)
	ret, err_msg = run_watch.communicate()
	print ("Restart Messages:\n%s" % ret + err_msg)


def parse_argument():
	"""
	This function just deal with the argument
	"""
	discovery_cfg = ""
	generate_ngs_cfg = ""
	parser = argparse.ArgumentParser()
	parser.add_argument(
		"-d", "--discovery_cfg", help="Config file for discovery")
	parser.add_argument(
		"-g", "--generate_ngs_cfg", help="Generated Nagios config file")
	args = parser.parse_args()
	if args.discovery_cfg:
		discovery_cfg = args.discovery_cfg
	
	if args.generate_ngs_cfg:
		generate_ngs_cfg = args.generate_ngs_cfg
	
	if discovery_cfg and generate_ngs_cfg:
		return discovery_cfg, generate_ngs_cfg
	else:
		parser.print_help()
		exit(1)
		return "", ""

def decrypt(ciphertext, tag = None):
	iv = key[:12]
	# Construct a Cipher object, with the key, iv, and additionally the
	# GCM tag used for authenticating the message.
	decryptor = Cipher(
		algorithms.AES(key),
		modes.GCM(iv, a2b_hex(tag)),
		backend=default_backend()
	).decryptor()

	# We put associated_data back in or the tag will fail to verify
	# when we finalize the decryptor.
	decryptor.authenticate_additional_data(associated_data)

	# Decryption gets us the authenticated plaintext.
	# If the tag does not match an InvalidTag exception will be raised.
	return decryptor.update(a2b_hex(ciphertext)) + decryptor.finalize()

def encrypt(plaintext):
	# Generate a random 96-bit IV.
	iv = key[:12]

	# Construct an AES-GCM Cipher object with the given key and a
	# randomly generated IV.
	encryptor = Cipher(
		algorithms.AES(key),
		modes.GCM(iv),
		backend=default_backend()
	).encryptor()

	# associated_data will be authenticated but not encrypted,
	# it must also be passed in on decryption.
	encryptor.authenticate_additional_data(associated_data)

	# Encrypt the plaintext and get the associated ciphertext.
	# GCM does not require padding.
	ciphertext = encryptor.update(plaintext) + encryptor.finalize()

	return (b2a_hex(ciphertext), b2a_hex(encryptor.tag))




if "__main__" == __name__:
	json_data, generate_ngs_cfg = parse_argument()
	f = nagios_discovery()
	host_ip = get_trap_info()
	new_dict = f.get_group_dict_via_host(host_ip, json_data)
	if not new_dict:
		print("Can not add host [%s] to nagios" % host_ip)
		exit(1)
	old_dict = f.load_config_file(generate_ngs_cfg)
	merge_dict = f.merge_dict(old_dict, new_dict)
	if merge_dict and new_dict:
		f.save_config_file(merge_dict, generate_ngs_cfg)
		reboot_nagios()
	else:
		print("This host [%s] has been monitored" % host_ip)

