#!/usr/bin/env python
#
# Copyright (C) 2018 Lenovo Corporation
#
# Time: 2018/03/07
# Description:  class for check_mk plugin installation

# this code just for compatible python2.7 and python3.5
from distutils.log import Log
# Init log level to infor
log = Log(2)
printf = log.info

from os import path, walk, remove
from common_lib import select_y_n
from parser_discovery_cfg import ParserDiscoveryCfg
from active_discovery import ActiveDiscovery

from sys import version_info as python_info
if python_info < (3, 0):
         lx_input = raw_input
else:
         lx_input = input

class NagiosCmkBase(object):

	def __init__(self, local_path = "", config_file = "", importfile = "", actv_dscry = False, pstv_dscry = False):
		self.site_path = ""
		self.local_path = local_path
		self.config_file = config_file
		self.importfile = importfile
		self.actv_dscry = actv_dscry
		self.pstv_dscry = pstv_dscry


	def _clear_etc(self):
		"""
		remove .cfg .mk and .wato files in $(local_path)/tools/etc/ before running this tool
		"""
		local_path = self.local_path
		if local_path.strip():
			etc_path = local_path.strip() + "tools/etc/"
			printf("Clear path %s" % etc_path)
			for root, dirs, files in walk(etc_path):
				for name in files:
					if name.endswith(('.cfg', '.mk', '.wato')):
						etc_file_name = path.join(root, name)
						remove(etc_file_name)
						printf("    remove file %s" % name)

    # Get the path of discovery config file
	def _get_discovery_cfg_file(self, depth=1, default_cfg="active_temp.cfg"):
		local_path = self.local_path
		cfg_file = lx_input("\nProvide the configuration file for discovery\n  (Press ENTER to continue with the default file %s):" % path.join(local_path, "template/%s" % default_cfg))
		if cfg_file:
			if not path.isfile(cfg_file):
				printf ("    %s is not a file" % cfg_file)
				if depth >= 3:
					printf ("    Default Discovery config file will be used (%s)\n" % path.join(local_path, "template/%s" % default_cfg))
					return path.join(local_path, "template/%s" % default_cfg)
				cfg_file = self._get_discovery_cfg_file(depth+1)
			else:
				printf ("    Discovery config file is %s\n" % cfg_file)
		else:
			cfg_file = path.join(local_path, 'template/%s' % default_cfg)
			printf ("    Discovery config file is %s\n" % cfg_file)
        
		return cfg_file


	def get_discovery_dict(self, discovery_cfg, is_active):
		"""
		For active discovery, this function will return xcc group dict
		For passive discovery, this function just return engineid group dict
		"""
		parsery_discovery = ParserDiscoveryCfg()
		discovery_raw_dict = parsery_discovery.get_raw_group_dict(discovery_cfg, True)

		if is_active:
			active_dscvy_instance = ActiveDiscovery()
			xcc_group_dict = active_dscvy_instance.active_discovery(discovery_raw_dict)
			return xcc_group_dict
		else:
			return discovery_raw_dict


	def get_import_dict(self, discovery_cfg):
		"""
		This function will return imported xcc group dict
		"""
		parsery_discovery = ParserDiscoveryCfg()
		discovery_raw_dict = parsery_discovery.get_raw_group_dict(discovery_cfg, True)
		return discovery_raw_dict

	def merge_dict(self, cfg_dict, discovery_dict):
		"""
		Merge the dict get from discovery and get from load nagios config file
		"""
		if not (cfg_dict and discovery_dict):
			cfg_dict.update(discovery_dict)
			return cfg_dict
		else:
			for key in discovery_dict:
				# merge same group information into one dict
				if key in cfg_dict:
					cfg_dict[key] = self._merge_common_dict(cfg_dict[key], discovery_dict[key])
				else:
					cfg_dict[key]=discovery_dict[key]
			return cfg_dict


	def _merge_common_dict(self, dict1, dict2):
		"""
		Merge dict1 and dict2, and merge lists "ip" and "service"
		"""
		for key in dict1:
			if (key in dict2) and (key in ["ip", "services"]):
				dict1[key] = list(set(dict1[key] + dict2[key]))
		for key in dict2:
			if key not in dict1:
				dict1[key] = dict2[key]
		return dict1


if "__main__" == __name__:
	pass

