#!/usr/bin/env python
# Copyright (C) 2018 Lenovo Corporation
#
# Time: 2018/03/07
# Description:  class for discovery Lenovo ThinkSytem Server


import argparse
import subprocess
import getpass
from multiprocessing.dummy import Pool as ThreadPool
from shutil import copy
from os import path, sep, walk
from common_lib import getiplist, get_connected_iplist
from common_lib import encrypt,decrypt

## ConfigParse is different from python2.7 and python3.5
from sys import version as python_version
if -1 != python_version.split(" ")[0].find("2.7"):
	import ConfigParser
	ConfigParser = ConfigParser.ConfigParser
else:
	import configparser
	ConfigParser = configparser.ConfigParser


class ParserDiscoveryCfg:

	def get_raw_group_dict(self, discover_cfg, is_active_discovery):
		"""
		Get raw group dict from discover config file
		"""
	
		# all, power,cpu,storage,temperature,fans,voltage,memory
		conf = ConfigParser()
		conf.read(discover_cfg)

		all_data_dict = {}
		# Get all the group name
		group_list = conf.sections()

		for group in group_list:
			items = dict(conf.items(group))

			ip_range, engineid, alias, user, authlevel, snmp_aprotocol, apassword, atag, snmp_pprotocol, ppassword, ptag, services = self._get_user_info_via_cfg(items)

			if ip_range == "" and is_active_discovery:
				print("    [ERROR]IP range is missing for active discovery (%s)" % group)
				continue

			if engineid == "" and not is_active_discovery:
				print("    [ERROR]Engine id is missing for passive discovery (%s)" % group)
				continue

			if user == "":
				print("    [ERROR]SNMPv3 User is missing for %s" % group)
				continue

			if alias == "":
				alias = group

			if services != "":
				services_list = services.split(",")
				services_list = [service.strip() for service in services_list if '' != service.strip()]
			else:
				services_list = ['all', 'power', 'cpu', 'storage', 'temperature', 'fans', 'voltage', 'memory']

			if is_active_discovery:
				## Get all ip list in the ip range
				all_ip_list = getiplist(ip_range)
				tmp_dict = dict(ip=all_ip_list, alias=alias, user = user, authlevel=authlevel, apassword=apassword, atag=atag, snmpaprotocol=snmp_aprotocol, ppassword=ppassword, ptag = ptag, snmppprotocol=snmp_pprotocol, services = services_list)
			else:
				engineid_list = engineid.split(",")
				engineid_list = [eng_id.strip() for eng_id in engineid_list]
				tmp_dict = dict(engineid=engineid_list, alias=alias, user = user, authlevel=authlevel, apassword=apassword, atag=atag, snmpaprotocol=snmp_aprotocol, ppassword=ppassword, ptag = ptag, snmppprotocol=snmp_pprotocol, services = services_list)

			all_data_dict[group] = tmp_dict

		return self.set_snmp_password(all_data_dict)


	def _get_user_info_via_cfg(self, group_dict):
		"""
		return:
			ip_range, engineid, alias, user, authlevel, snmp_aprotocol, apassword, snmp_pprotocol, ppassword
		"""
		ip_range, engineid, alias, user, authlevel, snmp_aprotocol, apassword, atag, snmp_pprotocol, ppassword, ptag, services = '','','','','','','',None,'','',None,''

		if 'ip' in group_dict.keys():
			ip_range = group_dict['ip']

		if 'engineid' in group_dict.keys():
			engineid = group_dict['engineid']

		if 'alias' in group_dict.keys():
			alias = group_dict['alias']

		if 'user' in group_dict.keys():
			user = group_dict['user']

		if 'authlevel' in group_dict.keys():
			authlevel = group_dict['authlevel']
		else:
			authlevel = "noAuthNoPriv"

		if 'snmpaprotocol' in group_dict.keys():
			snmp_aprotocol = group_dict['snmpaprotocol']

		if 'apassword' in group_dict.keys():
			apassword = group_dict['apassword']
			atag = None

		if 'snmppprotocol' in group_dict.keys():
			snmp_pprotocol = group_dict['snmppprotocol']

		if 'ppassword' in group_dict.keys():
			ppassword = group_dict['ppassword']
			ptag = None

		if 'services' in group_dict.keys():
			services = group_dict['services']
		
		return ip_range, engineid, alias, user, authlevel, snmp_aprotocol, apassword, atag, snmp_pprotocol, ppassword, ptag, services


	def set_snmp_password(self, raw_dict):
		for group, group_data in raw_dict.items():
			authlevel = group_data["authlevel"].lower()
			if "authpriv" == authlevel:
				print("Please input password for '%s' in group (%s)" % (group_data["user"],group))
				group_data["apassword"] = getpass.getpass("Authentication protocol password:")

				group_data["ppassword"] = getpass.getpass("Privacy protocol password:")

			elif "authnopriv" == authlevel:
				print("Please input password for '%s' in group (%s)" % (group_data["user"],group))
				group_data["apassword"] = getpass.getpass("Authentication protocol password:")
			elif "noauthnopriv" == authlevel:
				pass
		return raw_dict


if __name__ == "__main__":
	discovery_cfg = "../template/active_temp.cfg"
	tmp = ParserDiscoveryCfg()
	raw_group_dict = tmp.get_raw_group_dict(discovery_cfg, True)
	print raw_group_dict
	
